/* ------------------------------ ffth.c ------------------------------	*/
/*									*/
/* Author:	Eyal Lebedinsky (eyal@ise.canberra.edu.au)		*/
/* Date:	Jan 1994						*/
/* Version:	22 January 1994						*/
/*									*/
/* This program is released into the public domain.			*/
/*									*/
/*--------------------------------------------------------------------- */

#include <stdio.h>
#include <fcntl.h>
#include <math.h>
#include <graph.h>
#include <ctype.h>
#include <stdlib.h>

#include <conio.h>

extern void	FAR fft6 ();
extern void	FAR fft7 ();
extern void	FAR fft8 ();
extern void	FAR fft9 ();
extern void	FAR fft10();
extern int	FAR isqrt (long n);

#ifdef A2D
extern int FAR a2di (void), FAR a2d (void);
#else
#define a2d()	0
#define a2di()
#endif

#define mulf(x,y) (((long) (x) * (long) (y)) >> 15)
#define sqdf(x)   ((x) * (x))

#define MAXLEN		10
#define MAXSIZE		(2<<MAXLEN)

short  NEAR x[MAXSIZE];				/* fp(15,0)  */
short  NEAR qf[MAXSIZE/2+1];			/* fp(16,0)  */
static long tq[MAXSIZE/2+1];			/* fp(31,0)  */
static long av[MAXSIZE];			/* fp(31,0)  */
static int w[MAXSIZE];				/* fp(15,0)  */
static int old[MAXSIZE];

static char input[80];
static struct videoconfig vc;

#define XMAX		639
#define YMAX		347
#define LMARG		15
#define BMARG		332
#define HIGHT		256
#define	MY_BACK		0	/* background color */
#define	MY_FORE		15	/* foreground color */

static int	err = 0;

static char *name[] = {"ms", "10ms", "0.1s", "sec", "10sec"};

static int wadj = 0;

static long db2[6] =
	{65535L, 58408L, 52056L, 46395L, 41350L, 36853L};
static int	db[256];

static int FAR
idb_init ()
{
	register int	i, j;

	for (i = 0; i <48; ++i)
		db[256-i] = db2[i%6] >> (i/6 + 1);
	for (i = 48; i < 256; ++i)
		db[i] = 0;
}

static int FAR
idb (register int n)
{
	register int	i, j;

	if (n <= 0)
		return (0);

	if (n > 0x3fff) {
		n = n >> 7;
		i = 7 * 128;
	} else if (n > 0x1fff) {
		n = n >> 6;
		i = 6 * 128;
	} else if (n > 0x0fff) {
		n = n >> 5;
		i = 5 * 128;
	} else if (n > 0x07ff) {
		n = n >> 4;
		i = 4 * 128;
	} else if (n > 0x03ff) {
		n = n >> 3;
		i = 3 * 128;
	} else if (n > 0x01ff) {
		n = n >> 2;
		i = 2 * 128;
	} else if (n > 0x00ff) {
		n = n >> 1;
		i = 1 * 128;
	} else {
		i = 0 * 128;
	}
	if (i <= 0)
		return (n);
	return (i + db[n]);
}

static void FAR
set_pos (l, c)
int	l, c;
{
	(void)_settextposition (l, c);
/*   printf ("\33[%u;%uH", l, c);*/
}
   
static void FAR
clear_window ()
{
   _setviewport (LMARG+1, 1+1, LMARG+513-1, 36*2+1-1);
   _clearscreen (_GVIEWPORT);
   _setviewport (0, 0, XMAX, YMAX);
   set_pos (2, 3);
}

static char * FAR
set_winfunc (s, n)
int	s, n;
{
	register int	t;
	double	tt, pi;
	char *winfunc_name;

	switch (s) {
	case 0:			/* none */
		winfunc_name = "none";
		for (t = 0; t < n; ++t)
			w[t] = 127 * 256;
		wadj = 127 * 64 / 1.0;
		break;
	case 1:			/* rectangle */
		winfunc_name = "rectangle";
		w[0] = 0;
		for (t = 1; t < n; ++t)
			w[t] = 127 * 256;
		wadj = 127 * 64 / 1.0;
		break;
	case 2:			/* extended cosine */
		winfunc_name = "extended cosine";
		pi = 4 * atan (1.0);
		for (t = 0; t < n; ++t)
			if (t < n/8 || t > (7*n)/8)
				w[t] = (0.5 * (1.0 - cos ((8*pi*t)/n)))
					* 127 * 256;
			else
				w[t] = 127 * 256;
		wadj = 127 * 64 / 0.9;
		break;
	case 3:			/* sine */
		winfunc_name = "sine";
		pi = 4 * atan (1.0);
		for (t = 0; t < n; ++t)
			w[t] = sin ((pi*t)/n)
				* 127 * 256;
		wadj = 127 * 64 / 0.64;
		break;
	case 4:			/* triangle */
		winfunc_name = "triangle";
		for (t = 0; t < n; ++t) {
			if (t < n/2)
				w[t] = (2 * t * 128L * 256L) / n;
			else
				w[t] = (2 * (n - t - 1) * 128L * 256L) / n;
		}
		wadj = 127 * 64 / 0.5;
		break;
	case 5:			/* cosine**2 (hanning) */
		winfunc_name = "cosine^2 (hanning)";
		pi = 4 * atan (1.0);
		for (t = 0; t < n; ++t)
			w[t] = 0.5 * (1.0 - cos ((2*pi*t)/n))
				* 127 * 256;
		wadj = 127 * 64 / 0.5;
		break;
	case 6:			/* sine**3 */
		winfunc_name = "sine^3";
		pi = 4 * atan (1.0);
		for (t = 0; t < n; ++t) {
			tt = sin ((pi*t)/n);
			w[t] = tt * tt * tt
				* 127 * 256;
		}
		wadj = 127 * 64 / 0.42;
		break;
	case 7:			/* hamming */
		winfunc_name = "hamming";
		pi = 4 * atan (1.0);
		for (t = 0; t < n; ++t) {
			w[t] = (0.08 + 0.46 *(1.0 - cos ((2*pi*t)/n)))
				* 127 * 256;
		}
		wadj = 127 * 64 / 0.54;
		break;
	case 8:			/* cosine**4 */
		winfunc_name = "cosine^4";
		pi = 4 * atan (1.0);
		for (t = 0; t < n; ++t) {
			tt = 0.5 * (1.0 - cos ((2*pi*t)/n));
			w[t] = tt * tt
				* 127 * 256;
		}
		wadj = 127 * 64 / 0.36;
		break;
	case 9:			/* prazen */
		winfunc_name = "prazen";
		for (t = 0; t < n; ++t) {
			tt = (2.0 * t) / n - 1;
			if (t <= n/4 || t >= 3*(n/4)) {
				tt = 1.0 - fabs (tt);
				w[t] = 2.0 * tt * tt * tt
					* 127 * 256;
			} else {
				tt = fabs (tt);
				w[t] = (1.0 - 6.0 * tt * tt * (1.0 - tt))
					* 127 * 256;
			}
		}
		wadj = 127 * 64 / 0.37;
		break;
	default:
		winfunc_name = "unknown";
		break;
	}
	return (winfunc_name);
}

/* static unsigned char	buff[2*1024];*/
static unsigned char	*buff = 0;
static int		buff_size = 31*1024;

static FILE		*fin = 0;
static int		file = 0, count = 0, pos = 0;

static int
get_value ()
{
	int	v;

	if (!file)
		return (a2d ());

	while (pos >= count) {
		count = fread (buff, 1, buff_size, fin);
		if (count != buff_size)
			fseek (fin, 0L, SEEK_SET);
		pos = 0;
	}

	v = 16 * (buff[pos] + 256 * buff[pos+1] - 0x0800);
	pos += 2;

	return (v);
}

main (argc, argv)
int argc;
char *argv[]; {
   register short ii, *pp;
   int l, lll, n, n2, i, j, m, k, y, o, delay, white, black, grey, shift;
   int trigger, level, polarity, average, avcount, winfunc, factor;
   char *winfunc_name;
   long v, total, totcnt;
   char *b1, *b2, c, mode;
   unsigned short column, yy, dy;
   int xa = 40, xb = 2420, xc = 2400;	/* for sono */
   int xd = 5150, xe = 7500, xf = 5500;/* for osc, 'E' */
   int xg = 260, xh = 5800;		/* for osc, 'O' */
   int ya = 43, yb = 906, yc = 850;

   if (argc > 1)
	file = 1;

   /************************** parameters **********************************/

   l = 8;
   mode = 'O';
   delay = 0;
   white = 8;
   black = 0;
   grey = 8;
   shift = 0;
   level = trigger = 0;
   polarity = 1;
   average = avcount = 0;
   winfunc = 0;
   winfunc_name = "";
   total = totcnt = 0;
   wadj = 127 * 64 / 1.0;
   factor = 0;

   /************************** initialise **********************************/

   idb_init ();

   /************************** end initialise ******************************/

   if (_setvideomode (_VRES16COLOR) == 0) {
      printf ("cannot set VRES16COLOR mode.\n");
      exit (4);
   }

   _getvideoconfig (&vc);

   if (file) {
   	buff = (char *)malloc (buff_size);
   	if (buff == 0) {
		fprintf (stderr, "could not allocate file buffer\n");
		goto finish;
   	}
	fin = fopen (argv[1], "rb");
	if (fin == 0) {
		fprintf (stderr, "could not open input file %s\n", argv[1]);
		goto finish;
	}
   }

begin:

   _clearscreen (_GCLEARSCREEN);

   n = 1 << l; n2 = n >> 1;

   /*********************** screen layout ***********************************/

   set_pos (23, 3);
   printf ("White %2u Grey %u    ", white, grey);
   set_pos (24, 3);
   printf ("D%-4u L%u T%c%uL%-3d A%d  ",
	delay, l, polarity ? '+' : '-', trigger, level/256, average);

   set_pos (23, 23);
   printf ("F%u X%lu M%u       ", winfunc, total, factor);

   set_pos (24, 23);
   printf ("%s", winfunc_name);

   set_pos (23, 53);
   printf ("Cal Trig         Quit");
   set_pos (24, 53);
   printf ("Osc Son sPect Env   %c", mode);
   set_pos (25, 3);
   printf ("a=%u b=%u c=%u d=%u e=%u f=%u g=%u h=%u",
		xa, xb, xc, xd, xe, xf, xg, xh);
   printf (" i= %u j= %u k= %u",
		ya, yb, yc, winfunc_name);
   set_pos (2, 3);

   for (ii = 0; ii < n; ++ii)
      old[ii] = 0;
   for (ii = 0; ii <= n2; ++ii)
      tq[ii] = 0;

   _setcolor (MY_FORE);
   _moveto (LMARG,     36*2+1); 	/* waveform frame */
   _lineto (LMARG+513, 36*2+1);
   _lineto (LMARG+513, 1);
   _lineto (LMARG,     1);
   _lineto (LMARG,     36*2+1);

   if (mode == 'O' || mode == 'E') {	/* X axis */
      long step, jj;
      int xx;

      if (mode == 'E')
         jj = (delay * (long)xd + (delay ? xe : xf)) >> (l - 1);
      else
         jj = (delay * (long)xg + (delay ? xh : xf)) >> (l - 1);
      for (xx = 0, step = 1; jj > 150; jj /= 10) {
      	++xx;
      	step *= 10;
      }
      set_pos (6, 60);
      printf ("%-15s", name[xx]);

      for (i = 0;; ++i) {
      	 if (mode == 'E')
            jj = ((long)i << 16) / ((delay * (long)xd + (delay ? xe : xf)) >> 8);
      	 else
            jj = ((long)i << 16) / ((delay * (long)xg + (delay ? xh : xf)) >> 8);
	 j = (jj * step) >> l;
         if (j > 513 || i > 500)
	    break;
         _moveto (LMARG+j, (2*36+1));
         if      (i%10 == 0)		/* every 1.0 sec */
	    _lineto (LMARG+j, (2*36+1)+15);
         else if (i%5  == 0)		/* every 0.5 sec */
	    _lineto (LMARG+j, (2*36+1)+10);
         else				/* every 0.1 sec */
	    _lineto (LMARG+j, (2*36+1)+5);
      }
      clear_window ();
      goto again;
   }

   _moveto (LMARG, BMARG);		/* X axis */
   _lineto (XMAX,  BMARG);
   for (i = 0;; ++i) {
      j = (((long)i << 16) / (delay * (long)xa + (delay ? xb: xc))) << (8 - l);
      if (j > XMAX || i > 500)
	 break;
      _moveto (LMARG+j, BMARG);
      if      (i%10 == 0)		/* every 1.0 sec */
	 _lineto (LMARG+j, BMARG+15);
      else if (i%5  == 0)		/* every 0.5 sec */
	 _lineto (LMARG+j, BMARG+10);
      else				/* every 0.1 sec */
	 _lineto (LMARG+j, BMARG+5);
   }

   _moveto (LMARG, BMARG);		/* Y axis */
   _lineto (LMARG, BMARG-HIGHT);
   for (i = 0;; ++i) {
      j = ((long)i * (delay * (long)ya + (delay ? yb : yc))) >> 10 ;
      if (j >= 256)
	 break;
      _moveto (LMARG, BMARG-j);
      if      (i%50 == 0)
	 _lineto (LMARG-15,BMARG-j);	 /*   5 khz */
      else if (i%10 == 0)
	_lineto (LMARG-10, BMARG-j);	  /*   1 khz */
      else if (i%5  == 0)
	_lineto (LMARG-6, BMARG-j);	 /* 500 hz */
      else
	_lineto (LMARG-3, BMARG-j);	 /* 100 hz */
   }
   /*********************** end screen layout ****************************/

restart:

   column = LMARG+1;

again:

   while (kbhit ()) {
         gets (input);
	 if (sscanf (input, " %c ", &c) != 1)
	    continue;

	 switch (c = toupper (c)) {

	 case 'Q':
	    goto finish;

	 case 'P':
	 case 'O':
	 case 'S':
	 case 'E':
	    mode = c;
	    break;

	 case 'A':
	    if (input[1] == '=') {
	    	avcount = 0;
	    } else if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	             i < 0 || i > 255)
	        continue;
	    else {
		average = i;
		avcount = 0;
	    }
	    break;

	 case 'X':
	    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	             i < 0 || i > 32000)
	        continue;
	    else {
		total = i;
		totcnt = 0;
	    }
	    break;

	 case 'T':
	    if (input[1] == '+')
	    	polarity = 1;
	    else if (input[1] == '-')
	    	polarity = 0;
	    else if (toupper (input[1]) == 'L') {
		    if (sscanf (input, " %c%c %u ", &c, &c, &i) != 3 ||
		        i < -128 || i > 127)
		       continue;
		    level = i << 8;
	    } else {
		    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
		        i < 0 || i > 2)
		       continue;
		    trigger = i;
	    }
	    break;

	 case 'L':
	    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	        i < 6 || i > MAXLEN)
	       continue;
	    l = i;
	    n = 1 << l;
	    winfunc_name = set_winfunc (winfunc, n);
	    break;

	 case 'W':
	    if (sscanf (input, " %c %u ", &c, &i) != 2)
	       continue;
	    if (i >= 1 && i <= 15) {
	       white = i;
	       if (grey > white)
		  grey = white;
	       black = 8 - grey;
	       shift = 8 - white;
	    }
	    break;

	 case 'G':
	    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	        i < 0 || i > 8)
	       continue;
	    if (i > white)
		grey = white;
	    else
		grey = i;
	    black = 8 - grey;
	    break;

	 case 'D':
	    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	        i < 0 || i >= 10000)
	       continue;
	    delay = i;
	    break;

	 case 'F':
	    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	        i < 0 || i > 9)
	       continue;
	    winfunc = i;
	    winfunc_name = set_winfunc (winfunc, n);
	    break;

	 case 'M':
	    if (sscanf (input, " %c %u ", &c, &i) != 2 ||
	        i < 0 || i > 9)
	       continue;
	    factor = i;
	    break;

	 case 'C':
	    if (sscanf (input, " %c%c %u ", &c, &c, &i) != 3 || i < 0)
	       continue;
	 switch (c = toupper (c)) {
	    case 'A':
	       xa = i;
	       break;

	    case 'B':
	       xb = i;
	       break;

	    case 'C':
	       xc = i;
	       break;

	    case 'D':
	       xd = i;
	       break;

	    case 'E':
	       xe = i;
	       break;

	    case 'F':
	       xf = i;
	       break;

	    case 'G':
	       xg = i;
	       break;

	    case 'H':
	       xh = i;
	       break;

	    case 'I':
	       ya = i;
	       break;

	    case 'J':
	       yb = i;
	       break;

	    case 'K':
	       yc = i;
	       break;

	    default:
	       continue;
	    }
	    break;
	 }
	 goto begin;
   }

   if (average) {
      if (avcount >= average)
   	 avcount = 0;
      if (avcount == 0)
        for (i = 0; i < n; ++i)
	   av[i] = 0;
   }

   /*********************** input phase **********************************/

   if (!file) {
	if (0 != (err = a2di ()))
		goto finish;
   }

   if (trigger) {                  /* synch on trigger */
      int z;

      if (trigger == -1)
          goto again;
      if (trigger == 1)
          trigger = -1;
      if (polarity == 1) {
         while ((z=get_value ()) >= (level-2)) {
         	z = z * z;		/* delay for get_value () */
         }
         while ((z=get_value ()) <  level) {
         	z = z * z;
         }
      } else {
         while ((z=get_value ()) <= level)
         	z = z * z;
         while ((z=get_value ()) >  level)
         	z = z * z;
      }
   }

   if (delay) {
      if (mode == 'E') {
	   register long l;
	   register int ii, t;
	   register short *pp;

	   for (pp = x, i = n; i--;) {
	      for (l = 0, ii = delay; ii--;) {
      		if ((t = get_value ()) >= 0)
      		    l += t - 1;		/* avoid overflow */
      		else
      		    l -= t + 1;		/* avoid overflow */
      	      }
	      *pp++ = l / delay;
	   }
      } else {
	   register short *pp, *pn;
	   register int ii;

	   for (pn = (pp = x) + n; pp < pn;) {
	      for (ii = delay; ii--;)		 /* delay */
	         if (file)
	            get_value ();
	      *pp++ = get_value ();
	   }
      }
   } else {
	   register short *pp, *pn;
	   int z;

	   for (pn = (pp = x) + n; pp < pn;) {
	      *pp++ = get_value ();
	   }
   }

l1:
   if (average) {
	   register int i;

	   ++avcount;
	   for (i = 0; i < n; ++i) {
	      av[i] += x[i];
	      x[i] = av[i] / avcount;
	   }
	   if (avcount < average)
	      goto again;		/* don't show yet */
   }

   /*********************** end input phase ******************************/

   /*********************** show oscillogram *****************************/

   if (mode == 'E') {
      if (delay == 0)
         goto mode_o;
      clear_window ();
      if (l > 9) {
	 o = l - 9;
	 _moveto (LMARG+1, 32+36 - (x[0] >> 9));
	 for (ii = 1; ii < n; ++ii)
	    _lineto (LMARG+1 + (ii >> o), 32+36 - (x[ii] >> 9));
      } else {
	 o = 9 - l;
	 _moveto (LMARG+1, 32+36 - (x[0] >> 9));
	 for (ii = 1; ii < n; ++ii)
	    _lineto (LMARG+1 + (ii << o), 32+36 - (x[ii] >> 9));
      }
      goto again;
   }

   if (mode == 'O') {
      int xx, yold, ynew;

mode_o:
      if (l > 9)
	dy = 1,  lll = 1 << (l - 9);
      else
	lll = 1, dy  = 1 << (9 - l);

      yold = old[0];
      old[0] = ynew = x[0];
      xx = LMARG+1;
      for (ii = 1; ii < n; ii += lll, xx += dy) {
/* erase old line
*/
	_moveto (xx,    36 - (yold >> 10));
	yold = old[ii];
	_setcolor (MY_BACK);
	_lineto (xx+dy, 36 - (yold >> 10));

/* draw new line
*/
	_moveto (xx,    36 - (ynew >> 10));
	ynew = x[ii];
	_setcolor (MY_FORE);
	_lineto (xx+dy, 36 - (ynew >> 10));

	old[ii] = ynew;
      }
      goto again;
   }

   /*********************** end show oscillogram *************************/

   if (winfunc) {
   	for (i = 0; i < n; ++i)
   		x[i] = mulf (x[i], w[i]);
   }

   switch (l) {
   case 6:
	fft6 ();
	break;

   case 7:
	fft7 ();
	break;

   case 8:
	fft8 ();
	break;
   case 9:
	fft9 ();
	break;
   case 10:
	fft10 ();
	break;
   }
   for (ii = 0; ii <=n2; ++ii)
   	qf[ii] = idb (mulf (qf[ii] << (2 + factor), wadj));
/*   	qf[ii] = mulf (isqrt (qf[ii]) << (2 + factor), wadj);*/

   /*********************** show spectrogram *********************************/

   if (mode == 'P') {
#if 0
      if (total) {
      	 if (totcnt == total) {
      	    for (ii = 0; ii <= n2; ++ii)
      	       tq[ii] = 0;
      	    totcnt = 1;
      	 } else
      	    ++totcnt;
      }
#endif
      k = BMARG;

      if (l > 9)
	j = 1,   lll = 1 << (l - 9);
      else
	lll = 1, j   = 1 << (9 - l);

      for (ii = lll; ii <= n2; ii += lll) {
      	 if (total) {
#if 0
     	    v = tq[ii] += qf[ii];
      	    v /= totcnt;
#endif
     	    v = (tq[ii] = (tq[ii] * (total - 1)) / total + qf[ii]) / total;
      	 } else
      	    v = qf[ii];
	 if (white > 10)
	    v >>= (white - 10);
	 else
	    v <<= (10 - white);
	 if (v > XMAX)
	    v = XMAX;

	 k -= j;
	 if (old[ii] == v)
	 	continue;
	 _setcolor ((old[ii] < v) ? MY_FORE : MY_BACK);
	 _moveto (LMARG + old[ii], k);
	 _lineto (LMARG + v,       k);
	 old[ii] = v;
      }
       _setcolor (MY_FORE);
      goto again;
   }

   /*********************** end show spectrogram *****************************/

   /*********************** show sonogram ************************************/
   yy = BMARG-1;
   _moveto (column, yy);
   pp = qf;

   if (l > 9)
      dy = 1,  lll = 1 << (l - 9);
   else
      lll = 1, dy  = 1 << (9 - l);

   for (j = lll; j <= n2; j += lll) {
	m = *++pp;

	for (ii = 0; m >>= 1; ++ii)
		;

	if (ii > white)
	    ii = 8;
	else if ((ii += shift) <= black)
	    ii = 0;

	if (ii)
	    ii = ii*2 - 1;
	_setcolor (ii);
#if 0
	yy -= 1;
	_lineto (column, yy);
#endif
#if 0
	_setpixel (column, yy);
	_setpixel (column+1, yy);
	--yy;
	_setpixel (column, yy);
	_setpixel (column+1, yy);
	--yy;
#endif
#if 0
	_moveto (column+3, yy);
	_lineto (column+3, yy-1);
	_moveto (column, yy-1);
	_lineto (column, yy);
	yy -= 2;
#endif
	_moveto (column+1, yy);
	_lineto (column+1, yy-dy+1);
	_moveto (column,   yy-dy+1);
	_lineto (column,   yy);
	yy -= dy;
   }
   column += 2;
   if (column >= XMAX)
	goto restart;

   /*********************** end show sonogram ********************************/

   goto again;

finish:
   _setvideomode (_DEFAULTMODE);
   if (file) {
   	if (buff) {
	   	free (buff);
   		buff = 0;
   	}
   	if (fin) {
		fclose (fin);
		fin = 0;
	}
   }
   if (err) {
   	printf ("got error %d\n", err);
   	exit (1);
   }
   exit (0);
}

