/* ----------------------- ffto86a.c ----------------------------------	*/
/*									*/
/* Author:	Eyal Lebedinsky						*/
/* Date:	Jan 1994						*/
/* Version:	21 January 1994						*/
/*									*/
/* Called from fftg.c and generates Intel 80x86 .asm macro calls	*/
/* Breaks output into smaller files.					*/
/*									*/
/* This program is released into the public domain.			*/
/*									*/
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

static int	stmts = 0, file = 0, flen = 0, elen = 0;
static char	fname[256], ename[256];

void
start_fft (file_name, ep_name)
char	*file_name, *ep_name;
{
	if (file_name) {
		strcpy (ename, ep_name);
		strcpy (fname, file_name);
		strcat (fname, "aa.asm");
		flen = strlen (fname);
		file = 0;
	}
	freopen (fname, "wt", stdout);

	stmts = 0;
	printf (" INCLUDE ffts86a.mac\n");
	printf (" ffthead _TEXT%-2.2s,_%s\n", fname+flen-6, ename);
	stmts += 2;

	if (file_name) {		/* first time */
		printf (" fftenter\n");
		stmts += 1;
	}
}

void
end_fft (file_name, ep_name)
char	*file_name, *ep_name;
{
	printf (" fftexit _TEXT%-2.2s,_%s\n", fname+flen-6, ename);
	printf (" END\n");

	fclose (stdout);
}

void
break_fft (n)
int	n;
{
	char	eprev[256], fprev[10];

	stmts += n;
	if (stmts > 100) {
		strcpy (fprev, fname+flen-6);
		if (++fname[flen-5] > 'z') {		/* next file */
			fname[flen-5] = 'a';
			++fname[flen-6];
		}

		strcpy (eprev, ename);
		if (file++) {				/* next label */
			if (++ename[elen - 1] > 'z') {
				ename[elen - 1] = 'a';
				++ename[elen - 2];
			}
		} else {
			strcat (ename, "ab");
			elen = strlen (ename);
		}
		printf (" fftbreak _TEXT%-2.2s,_%s,_TEXT%-2.2s,_%s\n",
			fprev, eprev, fname+flen-6, ename);
		printf (" END\n");
		start_fft (NULL, NULL);
	}
}

void
fft1 (i1)
int	i1;
{
	printf (" fft1 %u\n", 2*i1);
	break_fft (1);
}

void
fft2 (i1, i2)
int	i1, i2;
{
	printf (" fft2 %u,%u\n", 2*i1, 2*i2);
	break_fft (1);
}

void
fft3 (i1, i3, i4)
int	i1, i3, i4;
{
	printf (" fft3 %u,%u,%u\n", 2*i1, 2*i3, 2*i4);
	break_fft (1);
}

void
fft4 (i1, i2, i3, i4)
int	i1, i2, i3, i4;
{
	printf (" fft4 %u,%u,%u,%u\n", 2*i1, 2*i2, 2*i3, 2*i4);
	break_fft (1);
}

void
fft5 (i1, i2, i3, i4, i5, i6, i7, i8, sm1, sp1, cc1, sm3, sp3, cc3, ind)
int	i1, i2, i3, i4, i5, i6, i7, i8, sm1, sp1, cc1, sm3, sp3, cc3, ind;
{
	printf (" fft5a %u,%u,0%04xH,0%04xH,0%04xH,%u\n",
		2*i3, 2*i7, sm1, sp1, cc1, ind);

	printf (" fft5b %u,%u,0%04xH,0%04xH,0%04xH,%u\n",
		2*i4, 2*i8, sm3, sp3, cc3, ind);

	printf (" fft5c %u,%u,%u,%u,%u,%u,%u,%u\n",
		2*i1, 2*i2, 2*i3, 2*i4, 2*i5, 2*i6, 2*i7, 2*i8);
	break_fft (3);
}

void
fft7 (i1, i2)
int	i1, i2;
{
	printf (" fft7 %u,%u\n", 2*i1, 2*i2);
	break_fft (1);
}

void
fft8 (i1, i2, i3)
int	i1, i2, i3;
{
	printf (" fft8 %u,%u,%u\n", 2*i1, 2*i2, 2*i3);
	break_fft (1);
}
