/* ------------------------- ffto86x.c -------------------------------- */
/*									*/
/* Author:	Eyal Lebedinsky						*/
/* Date:	Jan 1994						*/
/* Version:	19 February 1995					*/
/*									*/
/* called from fftg.c and generates a control table in assembly.	*/
/*									*/
/* This program is released into the public domain.			*/
/*									*/
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#define ENTRY_TYPE	0x2000

static int	stmts = 0, file = 0, flen = 0, elen = 0, done_fft = 0;
static char	fname[256] = "", ename[256] = "";

void
start_fft (file_name, ep_name)
char	*file_name, *ep_name;
{
	if (file_name) {
		strcpy (ename, ep_name);
		strcpy (fname, file_name);
		strcat (fname, "aa.asm");
		flen = strlen (fname);
		file = 0;
	}

	freopen (fname, "wt", stdout);
	printf (" INCLUDE ffts86x.mac\n");
	printf ("tstart t_%s,d_%s,_%s,l_%s\n", ename, ename, ename, ename);
	stmts = 2;
}

void
end_fft (file_name, ep_name)
char	*file_name, *ep_name;
{
	printf ("tend d_%s\n", 	ename);
	if (!done_fft) {
		printf ("tfft t_%s,d_%s,_%s,l_%s\n",
			ename, ename, ep_name, ep_name);

		done_fft = 1;
	}
	printf (" END\n");
	fclose (stdout);
}

void
break_fft (n)
int	n;
{
	char	eprev[256], fprev[10];

	stmts += n;
	if (stmts > 100) {
		strcpy (fprev, fname+flen-6);
		if (++fname[flen-5] > 'z') {		/* next file */
			fname[flen-5] = 'a';
			++fname[flen-6];
		}

		strcpy (eprev, ename);
		if (file++) {				/* next label */
			if (++ename[elen - 1] > 'z') {
				ename[elen - 1] = 'a';
				++ename[elen - 2];
			}
		} else {
			strcat (ename, "ab");
			elen = strlen (ename);
		}
		printf (" tbreak d_%s,d_%s,l_%s\n",
			eprev, ename, ename);
		if (!done_fft) {
			printf ("tfft t_%s,d_%s,_%s,l_%s\n",
				eprev, eprev, eprev, eprev);
			done_fft = 1;
		}

		printf (" END\n");
		start_fft (NULL, NULL);
	}
}

void
fft1 (i1)
int	i1;
{
	printf (" DW 0%04xH\n", 
		1*ENTRY_TYPE+2*i1);
	break_fft (1);
}

void
fft2 (i1, i2)
int	i1, i2;
{
	printf (" DW 0%04xH,%d\n", 
		2*ENTRY_TYPE+2*i1, 2*i2);
	break_fft (1);
}

void
fft3 (i1, i3, i4)
int	i1, i3, i4;
{
	printf (" DW 0%04xH,%d,%d\n", 
		3*ENTRY_TYPE+2*i1, 2*i3, 2*i4);
	break_fft (1);
}

void
fft4 (i1, i2, i3, i4)
int	i1, i2, i3, i4;
{
	printf (" DW 0%04xH,%d,%d,%d\n", 
		4*ENTRY_TYPE+2*i1, 2*i2, 2*i3, 2*i4);
	break_fft (1);
}

void
fft5 (i1, i2, i3, i4, i5, i6, i7, i8, sm1, sp1, cc1, sm3, sp3, cc3, ind)
int	i1, i2, i3, i4, i5, i6, i7, i8, sm1, sp1, cc1, sm3, sp3, cc3, ind;
{
	printf (" DW 0%04xH,%d,%d,%d,%d,%d,%d,%d\n",
		5*ENTRY_TYPE+2*i1, 2*i2, 2*i3, 2*i4, 2*i5, 2*i6, 2*i7, 2*i8);
	printf (" DW 0%04xH,0%04xH,0%04xH,0%04xH,0%04xH,0%04xH,%d\n",
		sm1, sp1, cc1, sm3, sp3, cc3, ind);
	break_fft (1);
}

void
fft7 (i1, i2)
int	i1, i2;
{
	printf (" DW 0%04xH,%d\n", 
		6*ENTRY_TYPE+2*i1, 2*i2);
	break_fft (1);
}

void
fft8 (i1, i2, i3)
int	i1, i2, i3;
{
	printf (" DW 0%04xH,%d,%d\n", 
		7*ENTRY_TYPE+2*i1, 2*i2, 2*i3);
	break_fft (1);
}
