/* ------------------------- fftsm.c ---------------------------------- */
/*									*/
/* Author:	Eyal Lebedinsky						*/
/* Date:	September 1991						*/
/* Version:	4 September 1991					*/
/*									*/
/* Called from the output of fftgm.					*/
/* It implements the interpreted condensed table technique.		*/
/*									*/
/* This program is released into the public domain.			*/
/*									*/
/*----------------------------------------------------------------------*/

extern short NEAR x[], NEAR qf[];

#define ENTRY_TYPE	0x2000
#define INDEX_MASK	(ENTRY_TYPE-1)
#define ENTRY_MASK	(~INDEX_MASK)

#define	X(i)	*(short *)(((char *)x)+i)
#define	QF(i)	*(short *)(((char *)qf)+i)

#define mulf(x,y) ((short)(((short)(x) * (long)(short)(y)) >> 16))

void FAR
FFT_NAME ()
{
	register short	*p;
	short		p0;

	p = fft_tab;
	for (;;) {
		p0 = *p & INDEX_MASK;
		switch (*(unsigned short *)p & ENTRY_MASK){
		case 1*ENTRY_TYPE: {
				X(p0) >>= 1;
			}
			p += 1;
			break;
		case 2*ENTRY_TYPE: {
				short	t0, t1;

				t0 = X(p0) >> 1;
				t1 = X(p[1]) >> 1;
				X(p0) = t0 + t1;
				X(p[1]) = t0 - t1;
			}
			p += 2;
			break;
		case 3*ENTRY_TYPE: {
				short	t1, t3, t4, tt;

				t1 = X(p0) >> 1;
				t3 = X(p[1]) >> 1;
				t4 = X(p[2]) >> 1;
				tt    = (t4 + t3) >> 1;
				X(p[2]) = (t4 - t3) >> 1;
				X(p[1]) =  t1 - tt;
				X(p0) =  t1 + tt;
			}
			p += 3;
			break;
		case 4*ENTRY_TYPE: {
				short	t1, t2, t3, t4, tt1, tt2;

				t1 = X(p0) >> 2;
				t2 = X(p[1]) >> 1;
				t3 = mulf (X(p[2]), 0x2d41);
				t4 = mulf (X(p[3]), 0x2d41);
				tt1 = t3 + t4;
				tt2 = t3 - t4;
				X(p[3]) =  t2 - tt1;
				X(p[2]) = -t2 - tt1;
				X(p[1]) =  t1 - tt2;
				X(p0) =  t1 + tt2;
			}
			p += 4;
			break;
		case 5*ENTRY_TYPE: {
				short	t1, t2, t3, t4, t5, t6;

				t5 = X(p[6]);
				t1 = mulf (t5, p[8]);
				t6 = X(p[2]);
				if (p[14])
					t6 >>= 1;
				t5 += t6;
				t2 = mulf (t6, p[9]);
				t5 = mulf (t5, p[10]);
				t1 += t5;
				t2 += t5;

				t5 = X(p[7]);
				t3 = mulf (t5, p[11]);
				t6 = X(p[3]);
				if (p[14])
					t6 >>= 1;
				t5 += t6;
				t4 = mulf (t6, p[12]);
				t5 = mulf (t5, p[13]);
				t3 += t5;
				t4 += t5;

				t5	= t3 - t1;
				t6	=  X(p[1]) >> 1;
				X(p[6]) = -t6 + t5;
				X(p[3]) =  t6 + t5;
				t5	= t2 + t4;
				t6	=  X(p[5]) >> 1;
				X(p[2]) = -t6 + t5;
				X(p[7]) =  t6 + t5;
				t5	= t1 + t3;
				t6	=  X(p0) >> 1;
				X(p[5]) =  t6 - t5;
				X(p0) =  t6 + t5;
				t5	= t2 - t4;
				t6	=  X(p[4]) >> 1;
				X(p[4]) =  t6 - t5;
				X(p[1]) =  t6 + t5;
			}
			p += 15;
			break;
		case 6*ENTRY_TYPE: {
				long	lt1;

				lt1 = X(p[1]);
				lt1 *= lt1;
				QF(p0) = (short)(lt1 >> 16);
			}
			p += 2;
			break;
		case 7*ENTRY_TYPE: {
				long	lt1, lt2;

				lt1 = X(p[1]);
				lt1 *= lt1;
				lt2 = X(p[2]);
				lt2 *= lt2;
				QF(p0) = (short)((lt1 + lt2) >> 16);
			}
			p += 3;
			break;
		default:
			return;
		}
	}
}
