/*--------------------------- timer.c --------------------------------- */
/*									*/
/* Author:	Eyal Lebedinsky (eyal@ise.canberra.edu.au)		*/
/* Date:	Jan 1994						*/
/* Version:	22 January 1994						*/
/*									*/
/* This one comes from FLy8, it simply reads the timer chip and then	*/
/* converts it to microseconds.						*/
/*									*/
/* This program is released into the public domain.			*/
/*									*/
/*--------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

typedef unsigned short	Ushort;
typedef unsigned int	Uint;
typedef unsigned long	Ulong;

#if defined(MSDOS) || defined(DJGPP)

#ifdef DJGPP
#include <dos.h>
#define inp(p) 		inportb(p)
#define outp(p,b)	outportb(p,b)
#else
#include <conio.h>
#endif

/* #include "pc8254.h" */

/* --------------------------------- pc8254.h ------------------------------- */

/* Definitions for the timer chip 8254. Used for timekeeping and speaker
 * control.
*/

#define CHANNEL_0	0x0040		/* system tick */
#define	CHANNEL_2	0x0042		/* speaker tone */
#define	COMMAND_REG	0x0043
#define WRITE_CH0	0x0036
#define WRITE_CH2	0x00b6
#define READ_SPECIAL	0x00c2
#define	PORT_B		0x0061		/* speaker on/off control */
#define	XTAL		1193182L
#define TIMER_MODES	0x000e
#define TIMER_MODE	0x0002
#define TIMER_OUT	0x0080
#define TIMER_PERIOD	0x0000ffffL	/* this is the standard */

/* --------------------------------- pc8254.h end---------------------------- */

/* These are the timer reading support functions.
*/
static Ushort NEAR
Disable (void)
{
#ifdef DJGPP
	return (disable ());
#else
	Ushort	flags;

	_asm {
		pushf
		pop	flags
		cli
	}
	return (flags);
#endif
}

static void NEAR
Enable (Ushort flags)
{
#ifdef DJGPP
	enable ();
#else
	_asm {
		push	flags
		popf
	}
#endif
}

static Ulong	timer_period = TIMER_PERIOD;


extern void FAR
set_timer (void)
{}

extern Ushort FAR
get_timer (void)
{
	Ushort		t, status, flags;

	do {
		flags = Disable ();
		outp (COMMAND_REG, READ_SPECIAL);
		status = (Ushort) inp (CHANNEL_0);	/* get status */
		t  = (Ushort)inp (CHANNEL_0);		/* low byte */
		t += (Ushort)(inp (CHANNEL_0) << 8);	/* high byte */
		Enable (flags);
	} while (0 == t);

	if ((status & TIMER_MODES) == TIMER_MODE*3) {
		t >>= 1;
		if (status & TIMER_OUT)
			t += (Ushort)((timer_period+1)/2);
	}
	return ((Ushort)(timer_period - t));		/* make it count up */
}

extern Ulong FAR
timer_micros (Ushort old, Ushort new)
{
	Ushort	ticks;

	if (new >= old)
		ticks = new - old;
	else
		ticks = (new+timer_period) - old;
	if (ticks > (Ushort)4000)
		return ((Ulong)ticks * (Ulong)4000L / (Ulong)(XTAL/(1000L/4L)));
	else
		return ((Ulong)ticks * (Ulong)1000000L / (Ulong)XTAL);
}

#else

#include <sys/times.h>

static clock_t	epoch = 0;
static int	started = 0;

extern void FAR
set_timer (void)
{
	started = 0;
}

extern Ushort FAR
get_timer (void)
{
	struct tms	tms;

	times (&tms);

	if (started)
		return ((Ushort)(tms.tms_utime - epoch));
	else {
		epoch = tms.tms_utime;
		started = 1;
		return (0);
	}
}

extern Ulong FAR
timer_micros (Ushort old, Ushort new)
{
	Ulong	interval, ref, rate;

	interval = (Ulong)new - (Ulong)old;
#ifdef CLK_TCK
	ref = CLK_TCK;
#else
#ifdef HZ
	ref = HZ;
#else
	ref = 60;
#endif
#endif
	for (rate = 1000000UL; interval > 0x7fffffffUL/rate; rate /= 2)
		ref /= 2;
	if (0 == ref)
		return (0);		/* too bad... */
	return (interval * rate / ref);
}

#endif
